@@ -1,5 +1,6 @@
 """The base Simon models"""
 
+import sys
 import warnings
 
 from .connection import get_database, pymongo_supports_mongoclient
@@ -521,9 +522,9 @@ class Model(object):
         """Saves the object to the database.
 
         When saving a new document for a model with ``auto_timestamp``
-        set to ``True``, unless already provided, ``created`` will be
-        added with the current datetime in UTC. ``modified`` will
-        always be set with the current datetime in UTC.
+        set to ``True``, ``created`` will be added with the current
+        datetime in UTC. ``modified`` will always be set with the
+        current datetime in UTC.
 
         If the model has the ``required_fields`` options set, a
         :class:`TypeError` will be raised if any of the fields have not
@@ -534,28 +535,45 @@ class Model(object):
         :type safe: bool.
         :raises: :class:`TypeError`
 
+        .. versionchanged:: 0.4.0
+           ``created`` is always added to inserted documents when
+           ``auto_timestamp`` is ``True``
+
         .. versionadded:: 0.1.0
         """
 
-        # Associate the current datetime (in UTC) with the created
-        # and modified fields. While Python can store datetimes with
-        # microseconds, BSON only supports milliseconds. Rather than
-        # having different data at the time of save, drop the precision
-        # from the Python datetime before associating it with the
-        # instance.
-        if self._meta.auto_timestamp:
-            now = current_datetime()
-            if not ('_id' in self._document or 'created' in self):
-                self._document['created'] = now
-            self._document['modified'] = now
-
         # Use a copy of the internal document so _id can be safely
         # removed.
         fields = self._document.copy()
         fields.pop('_id', None)
 
-        # Use upsert=True so new documents can be inserted.
-        self._update(fields, upsert=True, safe=safe)
+        # Associate the current datetime (in UTC) with the created and
+        # modified fields. created will only be added to documents that
+        # are being inserted. The values are associated with the copy of
+        # the internal document so that the internal document will be in
+        # a consistent state if _update() raises an exception.
+        if self._meta.auto_timestamp:
+            now = current_datetime()
+            if '_id' not in self._document:
+                fields['created'] = now
+            fields['modified'] = now
+
+        try:
+            # Use upsert=True so new documents can be inserted.
+            self._update(fields, upsert=True, safe=safe)
+        except:
+            # Raise the exception that was caught
+            e = sys.exc_info()
+            raise e[1], None, e[2]
+        else:
+            # Because _update() didn't raise an exception, the created
+            # and modified values can be associated with the internal
+            # document.
+            if self._meta.auto_timestamp:
+                if 'created' in fields:
+                    self._document['created'] = fields['created']
+                if 'modified' in fields:
+                    self._document['modified'] = fields['modified']
 
     def save_fields(self, fields, safe=False):
         """Saves only the specified fields.
@@ -833,6 +851,9 @@ class Model(object):
 
         # Which function are we calling?
         if not id:
+            # PyMongo's insert() method calls the argument doc_or_docs
+            # instead of document.
+            kwargs['doc_or_docs'] = kwargs.pop('document')
             f = self._meta.db.insert
         else:
             f = self._meta.db.update
@@ -845,9 +866,7 @@ class Model(object):
 
         # For atomic updates, make sure the updates find their way back
         # to the internal document.
-        # Right now this happens for all atomic updates, including when
-        # called from save_fields(). I'd like to turn that one off.
-        if is_atomic(fields):
+        if not use_internal and is_atomic(fields):
             # After updating the document in the database, the instance
             # needs to be updated as well. Depending on the size of the
             # document, it may be time consuming to reload the entire
