@@ -15,43 +15,31 @@ import mock
 from simon import Model, connection
 from simon.query import Q
 
+from .utils import ModelFactory
+
 AN_OBJECT_ID_STR = '50d4dce70ea5fae6fb84e44b'
 AN_OBJECT_ID = ObjectId(AN_OBJECT_ID_STR)
 
 
-class TestModel1(Model):
-    attribute = 2
-
-    class Meta:
-        collection = 'test-simon'
-        database = 'test-simon-mock'
-        field_map = {'x': 'y'}
-
-
-class TestModel2(Model):
-    def __str__(self):
-        return 'this is the __str__'
-
-    def __unicode__(self):
-        return u'this is the __unicode__'
+DefaultModel = ModelFactory('DefaultModel')
+MappedModel = ModelFactory('ModelFactory', field_map={'fake': 'real'})
+StringModel = ModelFactory('StringModel',
+                           __str__=lambda x: 'this is the __str__',
+                           __unicode__=lambda x: 'this is the __unicode__')
 
 
 class TestBase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
-        # Use a dummy connection so that these tests can be run even
-        # when there is no MongoDB server. In order to do that, the
-        # connection module needs _databases to be a dictionary--
-        # this would normally be done during the first call to
-        # connect(), but there's no guarantee that will have happened--
-        # and it needs the connection placed into it
-        if not isinstance(connection._databases, dict):
-            connection._databases = {}
+        with mock.patch('simon.connection.MongoClient'):
+            cls.connection = connection.connect('localhost', name='test-simon')
 
-        if 'test-simon-mock' not in connection._databases:
-            connection._databases['test-simon-mock'] = {
-                'test-simon': None,
-            }
+    def tearDown(self):
+        # Even though warnings are only checked in a few tests, it's a
+        # good idea to reset them here just in case there's an exception
+        # in one of the tests
+        warnings.resetwarnings()
+        warnings.simplefilter('ignore')
 
     @classmethod
     def tearDownClass(cls):
@@ -64,14 +52,14 @@ class TestBase(unittest.TestCase):
         """Test the `all()` method."""
 
         with mock.patch('simon.Model.find') as mock_find:
-            TestModel1.all()
+            DefaultModel.all()
 
             mock_find.assert_called_with()
 
     def test_contains(self):
         """Test the `__contains__()` method."""
 
-        m = TestModel1()
+        m = DefaultModel()
         m._document['a'] = 1
         m._document['b'] = 2
 
@@ -82,39 +70,36 @@ class TestBase(unittest.TestCase):
     def test_contains_field_map(self):
         """Test the `__contains__()` method with a mapped field."""
 
-        m = TestModel1()
-        m._document['y'] = 1
+        m = MappedModel()
+        m._document['real'] = 1
 
-        self.assertIn('x', m)
-        self.assertIn('y', m)
-        self.assertNotIn('x', m._document)
+        self.assertIn('fake', m)
+        self.assertIn('real', m)
+        self.assertNotIn('fake', m._document)
 
     def test_create(self):
         """Test the `create()` method."""
 
-        with nested(mock.patch.object(TestModel1, '_update'),
-                    mock.patch('simon.base.current_datetime'),
-                    ) as (_update, current_datetime):
-            current_datetime.return_value = 1
+        TestModel = ModelFactory('TestModel', auto_timestamp=False)
 
-            TestModel1.create(a=1)
+        with mock.patch.object(TestModel, '_update') as _update:
+            TestModel.create(a=1)
 
-            _update.assert_called_with({'a': 1, 'created': 1, 'modified': 1},
-                                       safe=False, upsert=True)
+            _update.assert_called_with({'a': 1}, safe=False, upsert=True)
 
     def test_db(self):
         ("Test that the `db` attribute is associated with classes and "
          "instances.")
 
-        self.assertTrue(hasattr(TestModel1._meta, 'db'))
+        self.assertTrue(hasattr(DefaultModel._meta, 'db'))
 
-        m = TestModel1()
+        m = DefaultModel()
         self.assertTrue(hasattr(m._meta, 'db'))
 
     def test_delattr(self):
         """Test the `__delattr__()` method."""
 
-        m = TestModel1()
+        m = DefaultModel()
         m._document['a'] = 1
         with self.assertRaises(AttributeError):
             del m.b
@@ -129,81 +114,78 @@ class TestBase(unittest.TestCase):
     def test_delattr_field_map(self):
         """Test the `__delattr__()` method with a mapped field."""
 
-        m = TestModel1()
-        m._document['y'] = 1
+        m = MappedModel()
+        m._document['real'] = 1
 
-        del m.x
-        self.assertNotIn('y', m._document)
+        del m.fake
+        self.assertNotIn('real', m._document)
 
     def test_eq(self):
         """Test the `__eq__()` method."""
 
-        # I'm using TestModel2 here because TestModel1 has crazy
+        # I'm using TestModel here because TestModel1 has crazy
         # settings.
 
-        class TestModel2Subclass1(TestModel2):
-            class Meta:
-                collection = 'some-other-collection'
+        Subclass1 = ModelFactory('Subclass1', spec=DefaultModel,
+                                 collection='some-other-collection')
 
-        class TestModel2Subclass2(TestModel2):
-            class Meta:
-                database = 'some-other-database'
+        Subclass2 = ModelFactory('Subclass2', spec=DefaultModel,
+                                 database='some-other-database')
 
-        class TestModel2Subclass3(TestModel2):
-            class Meta:
-                collection = 'testmodel2s'
+        Subclass3 = ModelFactory('Subclass3', spec=DefaultModel,
+                                 collection='defaultmodels')
 
         # Different classes shouldn't be equal
-        m1 = TestModel2(_id=1)
-        m2 = TestModel1(_id=1)
+        m1 = DefaultModel(_id=1)
+        m2 = MappedModel(_id=1)
         self.assertFalse(m1 == m2)
         self.assertFalse(m2 == m1)
 
         # Even a subclass shouldn't be equal ...
         # ... when the collection is different
-        m1 = TestModel2(_id=3)
-        m2 = TestModel2Subclass1(_id=3)
+        m1 = DefaultModel(_id=3)
+        m2 = Subclass1(_id=3)
         self.assertFalse(m1 == m2)
         self.assertFalse(m2 == m1)
 
         # ... when the database is different
-        m1 = TestModel2(_id=4)
-        m2 = TestModel2Subclass2(_id=4)
+        m1 = DefaultModel(_id=4)
+        m2 = Subclass2(_id=4)
         self.assertFalse(m1 == m2)
         self.assertFalse(m2 == m1)
 
         # Unless they use the same database and collection
-        m1 = TestModel2(_id=1)
-        m2 = TestModel2Subclass3(_id=1)
+        m1 = DefaultModel(_id=1)
+        m2 = Subclass3(_id=1)
         self.assertTrue(m1 == m2)
         self.assertTrue(m2 == m1)
 
         # No _id's shouldn't be equal
-        m1 = TestModel2()
-        m2 = TestModel2()
+        m1 = DefaultModel()
+        m2 = DefaultModel()
         self.assertFalse(m1 == m2)
         self.assertFalse(m2 == m1)
 
         # Only one _id shouldn't be equal
-        m1 = TestModel2(_id=1)
-        m2 = TestModel2()
+        m1 = DefaultModel(_id=1)
+        m2 = DefaultModel()
         self.assertFalse(m1 == m2)
         self.assertFalse(m2 == m1)
 
         # Different _id's shouldn't be equal
-        m1 = TestModel2(_id=1)
-        m2 = TestModel2(_id=2)
+        m1 = DefaultModel(_id=1)
+        m2 = DefaultModel(_id=2)
         self.assertFalse(m1 == m2)
         self.assertFalse(m2 == m1)
 
         # The same _id should be equal
-        m1 = TestModel2(_id=1)
-        m2 = TestModel2(_id=1)
+        m1 = DefaultModel(_id=1)
+        m2 = DefaultModel(_id=1)
         self.assertTrue(m1 == m2)
         self.assertTrue(m2 == m1)
 
         # And just for the heck of it...
-        m1 = TestModel2(_id=1)
+        m1 = DefaultModel(_id=1)
         self.assertFalse(m1 == 'abc')
         self.assertFalse('abc' == m1)
 
@@ -212,15 +194,12 @@ class TestBase(unittest.TestCase):
 
         warnings.simplefilter('error', DeprecationWarning)
 
-        with mock.patch.object(TestModel1, '_find'):
+        with mock.patch.object(DefaultModel, '_find'):
             with self.assertRaises(DeprecationWarning) as e:
-                TestModel1.find(Q(a=1), Q(b=2))
-
-        warnings.resetwarnings()
-        warnings.simplefilter('default')
+                DefaultModel.find(Q(a=1), Q(b=2))
 
         expected = 'qs has been deprecated. Please use q instead.'
-        actual = e.exception.message
+        actual = str(e.exception)
         self.assertEqual(actual, expected)
 
     def test_get_deprecationwarning(self):
@@ -228,28 +207,25 @@ class TestBase(unittest.TestCase):
 
         warnings.simplefilter('error', DeprecationWarning)
 
-        with mock.patch.object(TestModel1, '_find'):
+        with mock.patch.object(DefaultModel, '_find'):
             with self.assertRaises(DeprecationWarning) as e:
-                TestModel1.get(Q(a=1), Q(b=2))
-
-        warnings.resetwarnings()
-        warnings.simplefilter('default')
+                DefaultModel.get(Q(a=1), Q(b=2))
 
         expected = 'qs has been deprecated. Please use q instead.'
-        actual = e.exception.message
+        actual = str(e.exception)
         self.assertEqual(actual, expected)
 
     def test_get_or_create_create(self):
         """Test the `get_or_create()` method for creating documents."""
 
-        with nested(mock.patch.object(TestModel1, 'get'),
-                    mock.patch.object(TestModel1, 'create'),
+        with nested(mock.patch.object(DefaultModel, 'get'),
+                    mock.patch.object(DefaultModel, 'create'),
                     ) as (get, create):
-            get.side_effect = TestModel1.NoDocumentFound
+            get.side_effect = DefaultModel.NoDocumentFound
 
             create.return_value = mock.Mock()
 
-            m, created = TestModel1.get_or_create(_id=AN_OBJECT_ID)
+            m, created = DefaultModel.get_or_create(_id=AN_OBJECT_ID)
 
             get.assert_called_with(_id=AN_OBJECT_ID)
             # Because get_or_create() is being called without explicity
@@ -262,10 +238,10 @@ class TestBase(unittest.TestCase):
     def test_get_or_create_get(self):
         """Test the `get_or_create()` method for getting documents."""
 
-        with mock.patch.object(TestModel1, 'get') as get:
+        with mock.patch.object(DefaultModel, 'get') as get:
             get.return_value = mock.Mock()
 
-            m, created = TestModel1.get_or_create(_id=AN_OBJECT_ID)
+            m, created = DefaultModel.get_or_create(_id=AN_OBJECT_ID)
 
             get.assert_called_with(_id=AN_OBJECT_ID)
 
@@ -274,7 +250,10 @@ class TestBase(unittest.TestCase):
     def test_getattr(self):
         """Test the `__getattr__()` method."""
 
-        m = TestModel1()
+        class TestModel(Model):
+            attribute = 2
+
+        m = TestModel()
 
         m._document['a'] = 1
         self.assertEqual(m.a, 1)
@@ -295,18 +274,18 @@ class TestBase(unittest.TestCase):
     def test_getattr_field_map(self):
         """Test the `__getattr__()` method with a mapped field."""
 
-        m = TestModel1()
+        m = MappedModel()
 
-        m._document['y'] = 1
+        m._document['real'] = 1
 
-        self.assertEqual(getattr(m, 'x'), 1)
+        self.assertEqual(getattr(m, 'fake'), 1)
 
     def test_increment(self):
         """Test the `increment()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.increment('a')
 
             _update.assert_called_with({'$inc': {'a': 1}}, safe=False)
@@ -318,9 +297,9 @@ class TestBase(unittest.TestCase):
     def test_increment_multiple(self):
         """Test the `increment()` method with multiple fields."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.increment(a=1, b=2)
 
             _update.assert_called_with({'$inc': {'a': 1, 'b': 2}}, safe=False)
@@ -328,7 +307,7 @@ class TestBase(unittest.TestCase):
     def test_increment_valueerror(self):
         """Test that `increment()` raises `ValueError`."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
         with self.assertRaises(ValueError):
             m.increment()
@@ -337,7 +316,7 @@ class TestBase(unittest.TestCase):
         """Test the `__init__()` method."""
 
         fields = {'a': 1}
-        m = TestModel1(**fields)
+        m = DefaultModel(**fields)
         self.assertTrue(all(getattr(m, k) == v for k, v in fields.items()))
 
         self.assertIsInstance(m._document, dict)
@@ -346,9 +325,9 @@ class TestBase(unittest.TestCase):
         """Test the `__ne__()` method."""
 
         # __ne__() should call __eq__(), so check that first
-        with mock.patch.object(TestModel1, '__eq__') as __eq__:
-            m1 = TestModel1(_id=1)
-            m2 = TestModel1(_id=1)
+        with mock.patch.object(DefaultModel, '__eq__') as __eq__:
+            m1 = DefaultModel(_id=1)
+            m2 = DefaultModel(_id=1)
 
             m1 != m2
 
@@ -356,14 +335,14 @@ class TestBase(unittest.TestCase):
 
         # __ne__() should return the opposite of __eq__(), so perform
         # a couple of simple comparisons just to make sure
-        m1 = TestModel1(_id=1)
-        m2 = TestModel1(_id=1)
+        m1 = DefaultModel(_id=1)
+        m2 = DefaultModel(_id=1)
         self.assertNotEqual(m1 == m2, m1 != m2)
         self.assertNotEqual(m2 == m1, m2 != m1)
         self.assertEqual(m1 != m2, m2 != m1)
 
-        m1 = TestModel1(_id=1)
-        m2 = TestModel1(_id=2)
+        m1 = DefaultModel(_id=1)
+        m2 = DefaultModel(_id=2)
         self.assertNotEqual(m1 == m2, m1 != m2)
         self.assertNotEqual(m2 == m1, m2 != m1)
         self.assertEqual(m1 != m2, m2 != m1)
@@ -371,9 +350,9 @@ class TestBase(unittest.TestCase):
     def test_raw_update(self):
         """Test the `raw_update()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.raw_update({'$set': {'a': 1}})
 
             _update.assert_called_with({'$set': {'a': 1}}, safe=False)
@@ -381,9 +360,9 @@ class TestBase(unittest.TestCase):
     def test_remove_fields(self):
         """Test the `remove_fields()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.remove_fields('a')
 
             _update.assert_called_with({'$unset': {'a': 1}}, safe=False)
@@ -391,9 +370,9 @@ class TestBase(unittest.TestCase):
     def test_remove_fields_multiple(self):
         """Test the `remove_fields()` method with multiple fields."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.remove_fields(('a', 'b'))
 
             _update.assert_called_with({'$unset': {'a': 1, 'b': 1}},
@@ -402,39 +381,52 @@ class TestBase(unittest.TestCase):
     def test_repr(self):
         """Test the `__repr__()` method."""
 
-        m = TestModel1()
+        m = DefaultModel()
 
-        expected = '<TestModel1: TestModel1 object>'
+        expected = '<DefaultModel: DefaultModel object>'
         actual = '{0!r}'.format(m)
         self.assertEqual(actual, expected)
 
-        m2 = TestModel2()
+        m2 = StringModel()
 
-        expected = '<TestModel2: this is the __str__>'
+        expected = '<StringModel: this is the __str__>'
         actual = '{0!r}'.format(m2)
         self.assertEqual(actual, expected)
 
     def test_save(self):
         """Test the `save()` method."""
 
-        m = TestModel1(a=1)
+        TestModel = ModelFactory('TestModel', auto_timestamp=False)
 
-        with nested(mock.patch.object(TestModel1, '_update'),
-                    mock.patch('simon.base.current_datetime'),
-                    ) as (_update, current_datetime):
-            current_datetime.return_value = 1
+        m = TestModel(a=1)
 
+        with mock.patch.object(TestModel, '_update') as _update:
             m.save()
 
-            _update.assert_called_with({'a': 1, 'created': 1, 'modified': 1},
-                                       safe=False, upsert=True)
+            _update.assert_called_with({'a': 1}, safe=False, upsert=True)
+
+    def test_save_exception(self):
+        """Test that `save()` raises the exception it catches."""
+
+        m = DefaultModel(a=1)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            _update.side_effect = ValueError
+
+            with self.assertRaises(ValueError):
+                m.save()
+
+            _update.side_effect = TypeError
+
+            with self.assertRaises(TypeError):
+                m.save()
 
     def test_save_fields(self):
         """Test the `save_fields()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID, a=1)
+        m = DefaultModel(_id=AN_OBJECT_ID, a=1)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.save_fields('a')
 
             _update.assert_called_with({'$set': {'a': 1}}, use_internal=True,
@@ -443,22 +435,22 @@ class TestBase(unittest.TestCase):
     def test_save_fields_attributeerror(self):
         """Test that `save_fields()` raises `AttributeError`."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
         with self.assertRaises(AttributeError) as e:
             m.save_fields('a')
 
-        expected = ("The 'TestModel1' object does not have all of the "
+        expected = ("The 'DefaultModel' object does not have all of the "
                     "specified fields.")
-        actual = e.exception.message
+        actual = str(e.exception)
         self.assertEqual(actual, expected)
 
     def test_save_fields_multiple(self):
         """Test the `save_fields()` method with multiple fields."""
 
-        m = TestModel1(_id=AN_OBJECT_ID, a=1, b=2)
+        m = DefaultModel(_id=AN_OBJECT_ID, a=1, b=2)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.save_fields(('a', 'b'))
 
             _update.assert_called_with({'$set': {'a': 1, 'b': 1}},
@@ -467,25 +459,53 @@ class TestBase(unittest.TestCase):
     def test_save_timestamps(self):
         """Test that `save()` properly handles adding timestamps."""
 
-        with mock.patch.object(TestModel1, '_update'):
+        with mock.patch.object(DefaultModel, '_update'):
             # Insert
-            m1 = TestModel1()
+            m1 = DefaultModel()
             m1.save()
 
             self.assertIsInstance(m1._document['created'], datetime)
             self.assertIsInstance(m1._document['modified'], datetime)
 
             # Update
-            m2 = TestModel1(_id=AN_OBJECT_ID)
+            m2 = DefaultModel(_id=AN_OBJECT_ID, modified=1)
             m2.save()
 
             self.assertNotIn('created', m2._document)
             self.assertIsInstance(m2._document['modified'], datetime)
 
+    def test_save_timestamps_reset(self):
+        ("Test that `save()` properly resets timestamps after an "
+         "exception.")
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            _update.side_effect = Exception
+
+            m1 = DefaultModel()
+            try:
+                m1.save()
+            except:
+                pass  # Exception will be raised for this test
+
+            self.assertNotIn('created', m1._document)
+            self.assertNotIn('modified', m1._document)
+
+            m2 = DefaultModel(_id=AN_OBJECT_ID, created=1, modified=2)
+            try:
+                m2.save()
+            except:
+                pass  # Exception will be raised for this test
+
+            self.assertEqual(m2._document['created'], 1)
+            self.assertEqual(m2._document['modified'], 2)
+
     def test_setattr(self):
         """Test the `__setattr__()` method."""
 
-        m = TestModel1()
+        class TestModel(Model):
+            attribute = 2
+
+        m = TestModel()
         m._document['a'] = 1
         self.assertNotIn('b', m._document)
         with self.assertRaises(AttributeError):
@@ -518,22 +538,22 @@ class TestBase(unittest.TestCase):
     def test_setattr_field_map(self):
         """Test the `__setattr__()` method with a mapped field."""
 
-        m = TestModel1()
-        setattr(m, 'x', 1)
+        m = MappedModel()
+        setattr(m, 'fake', 1)
 
-        self.assertIn('y', m._document)
-        self.assertEqual(m._document['y'], 1)
+        self.assertIn('real', m._document)
+        self.assertEqual(m._document['real'], 1)
 
     def test_str(self):
         """Test the `__str__()` method."""
 
-        m = TestModel1()
+        m = DefaultModel()
 
-        expected = 'TestModel1 object'
+        expected = 'DefaultModel object'
         actual = '{0!s}'.format(m)
         self.assertEqual(actual, expected)
 
-        m2 = TestModel2()
+        m2 = StringModel()
 
         expected = 'this is the __str__'
         actual = '{0!s}'.format(m2)
@@ -542,13 +562,14 @@ class TestBase(unittest.TestCase):
     def test_unicode(self):
         """Test the `__unicode__()` method."""
 
-        m = TestModel1()
+        m = DefaultModel()
 
-        expected = u'TestModel1 object'
+        expected = u'DefaultModel object'
         actual = u'{0}'.format(m)
         self.assertEqual(actual, expected)
 
-        m2 = TestModel2()
+        m2 = StringModel()
+        print m2.__str__, str(m2)
 
         expected = u'this is the __unicode__'
         actual = u'{0}'.format(m2)
@@ -557,9 +578,9 @@ class TestBase(unittest.TestCase):
     def test_update(self):
         """Test the `update()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1, '_update') as _update:
+        with mock.patch.object(DefaultModel, '_update') as _update:
             m.update(a=1)
 
             _update.assert_called_with({'$set': {'a': 1}}, safe=False)
