@@ -11,7 +11,9 @@ from bson import ObjectId
 import mock
 import pymongo
 
-from simon import Model, connection, query
+from simon import connection, query
+
+from .utils import ModelFactory
 
 AN_OBJECT_ID_STR = '50d4dce70ea5fae6fb84e44b'
 AN_OBJECT_ID = ObjectId(AN_OBJECT_ID_STR)
@@ -24,45 +26,9 @@ else:
     wc_on = {'safe': True}
     wc_off = {'safe': False}
 
-
-class TestModel1(Model):
-    class Meta:
-        collection = 'test-simon'
-        database = 'test-simon'
-        field_map = {'id': '_id', 'fake': 'real'}
-
-
-class TestModel2(Model):
-    class Meta:
-        auto_timestamp = False
-        collection = 'test-simon'
-        database = 'test-simon'
-
-
-class TestModel3(Model):
-    class Meta:
-        collection = 'test-simon'
-        database = 'test-simon'
-        safe = False
-        sort = 'a'
-
-
-class TestModel4(Model):
-    class Meta:
-        collection = 'test-simon'
-        database = 'test-simon'
-        sort = '-a'
-
-
-class TestModel5(Model):
-    class Meta:
-        auto_timestamp = False
-        required_fields = ('a', 'b')
-
-
-class TestModel6(Model):
-    class Meta:
-        required_fields = 'a.b'
+DefaultModel = ModelFactory('DefaultModel')
+MappedModel = ModelFactory('MappedModel', field_map={'fake': 'real'})
+RequiredModel = ModelFactory('RequiredModel', required_fields=('a', 'b'))
 
 
 class TestDatabase(unittest.TestCase):
@@ -83,7 +49,7 @@ class TestDatabase(unittest.TestCase):
     def test___init__(self):
         """Test the `__init__()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID, a=1, b=2)
+        m = DefaultModel(_id=AN_OBJECT_ID, a=1, b=2)
 
         self.assertEqual(m._id, AN_OBJECT_ID)
         self.assertEqual(m._document['_id'], AN_OBJECT_ID)
@@ -97,9 +63,9 @@ class TestDatabase(unittest.TestCase):
     def test_delete(self):
         """Test the `delete()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1._meta.db, 'remove') as remove:
+        with mock.patch.object(DefaultModel._meta.db, 'remove') as remove:
             m.delete()
 
             remove.assert_called_with({'_id': AN_OBJECT_ID}, **wc_on)
@@ -107,7 +73,7 @@ class TestDatabase(unittest.TestCase):
     def test_delete_typeerror(self):
         """Test that `delete()` raises `TypeError`."""
 
-        m = TestModel1(a=1, b=2)
+        m = DefaultModel(a=1, b=2)
 
         with self.assertRaises(TypeError):
             m.delete()
@@ -115,54 +81,56 @@ class TestDatabase(unittest.TestCase):
     def test_delete_write_concern(self):
         """Test that `delete()` respects write concern."""
 
-        with mock.patch.object(TestModel3._meta.db, 'remove') as remove:
-            m = TestModel3(_id=AN_OBJECT_ID)
+        UnsafeModel = ModelFactory('UnsafeModel', safe=False)
+
+        with mock.patch.object(UnsafeModel._meta.db, 'remove') as remove:
+            m = UnsafeModel(_id=AN_OBJECT_ID)
             m.delete()
             remove.assert_called_with({'_id': AN_OBJECT_ID}, **wc_off)
 
             # m needs to be given its _id again because delete() strips
             # it to prevent attempts to resave a deleted document
-            m = TestModel3(_id=AN_OBJECT_ID)
+            m = UnsafeModel(_id=AN_OBJECT_ID)
             m.delete(safe=False)
             remove.assert_called_with({'_id': AN_OBJECT_ID}, **wc_off)
 
-            m = TestModel3(_id=AN_OBJECT_ID)
+            m = UnsafeModel(_id=AN_OBJECT_ID)
             m.delete(safe=True)
             remove.assert_called_with({'_id': AN_OBJECT_ID}, **wc_on)
 
     def test__find(self):
         """Test the `_find()` method."""
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'find'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'find'),
                     mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
             find.return_value = QuerySet
 
-            TestModel1._find(_id=AN_OBJECT_ID)
+            DefaultModel._find(_id=AN_OBJECT_ID)
 
             find.assert_called_with({'_id': AN_OBJECT_ID})
 
     def test__find_field_map(self):
         """Test the `_find()` method with a name in `field_map`."""
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'find'),
+        with nested(mock.patch.object(MappedModel._meta.db, 'find'),
                     mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
             find.return_value = QuerySet
 
-            TestModel1._find(fake=1)
+            MappedModel._find(fake=1)
 
             find.assert_called_with({'real': 1})
 
     def test__find_find_one(self):
         """Test the `_find()` method with `find_one`."""
 
-        with mock.patch.object(TestModel1._meta.db, 'find') as find:
+        with mock.patch.object(DefaultModel._meta.db, 'find') as find:
             QuerySet = mock.MagicMock(spec=query.QuerySet)
             QuerySet.__getitem__.return_value = {'_id': AN_OBJECT_ID}
             QuerySet.count.return_value = 1
 
             find.return_value = QuerySet
 
-            m = TestModel1._find(find_one=True, _id=AN_OBJECT_ID)
+            m = DefaultModel._find(find_one=True, _id=AN_OBJECT_ID)
 
             find.assert_called_with({'_id': AN_OBJECT_ID})
 
@@ -171,18 +139,18 @@ class TestDatabase(unittest.TestCase):
     def test__find_find_one_multipledocumentsfound(self):
         """Test that `_find()` raises `MultipleDocumentsFound`."""
 
-        with mock.patch.object(TestModel1._meta.db, 'find') as find:
+        with mock.patch.object(DefaultModel._meta.db, 'find') as find:
             QuerySet = mock.MagicMock(spec=query.QuerySet)
             QuerySet.count.return_value = 2
 
             find.return_value = QuerySet
-            with self.assertRaises(TestModel1.MultipleDocumentsFound) as e:
-                TestModel1._find(find_one=True, _id=AN_OBJECT_ID)
+            with self.assertRaises(DefaultModel.MultipleDocumentsFound) as e:
+                DefaultModel._find(find_one=True, _id=AN_OBJECT_ID)
 
-            actual = e.exception.message
+            actual = str(e.exception)
 
             # Check the exception for the pieces
-            expected = "The query returned more than one 'TestModel1'."
+            expected = "The query returned more than one 'DefaultModel'."
             self.assertIn(expected, actual)
 
             expected = "It returned 2!"
@@ -203,90 +171,93 @@ class TestDatabase(unittest.TestCase):
     def test__find_find_one_nodocumentfound(self):
         """Test that `_find()` raises `NoDocumentFound`."""
 
-        with mock.patch.object(TestModel1._meta.db, 'find') as find:
+        with mock.patch.object(DefaultModel._meta.db, 'find') as find:
             QuerySet = mock.MagicMock(spec=query.QuerySet)
             QuerySet.count.return_value = 0
 
             find.return_value = QuerySet
-            with self.assertRaises(TestModel1.NoDocumentFound) as e:
-                TestModel1._find(find_one=True, _id=AN_OBJECT_ID)
+            with self.assertRaises(DefaultModel.NoDocumentFound) as e:
+                DefaultModel._find(find_one=True, _id=AN_OBJECT_ID)
 
-            expected = "'TestModel1' matching query does not exist."
-            actual = e.exception.message
+            expected = "'DefaultModel' matching query does not exist."
+            actual = str(e.exception)
             self.assertEqual(actual, expected)
 
     def test__find_nested_field(self):
         """The the `_find()` method with an embedded document."""
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'find'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'find'),
                     mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
             find.return_value = QuerySet
 
-            TestModel1._find(a__b=1)
+            DefaultModel._find(a__b=1)
 
             find.assert_called_with({'a.b': 1})
 
     def test__find_objectid_string(self):
         """Test the `_find()` method with a string `_id`."""
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'find'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'find'),
                     mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
             find.return_value = QuerySet
 
-            TestModel1._find(_id=AN_OBJECT_ID_STR)
+            DefaultModel._find(_id=AN_OBJECT_ID_STR)
 
             find.assert_called_with({'_id': AN_OBJECT_ID})
 
     def test__find_q(self):
         """Test the `_find()` method with a `Q` object."""
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'find'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'find'),
                     mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
             find.return_value = QuerySet
 
-            TestModel1._find(query.Q(a=1), _id=AN_OBJECT_ID)
+            DefaultModel._find(query.Q(a=1), _id=AN_OBJECT_ID)
 
             find.assert_called_with({'_id': AN_OBJECT_ID, 'a': 1})
 
     def test__find_q_alone(self):
         """Test the `_find()` method with nothing but a `Q` object."""
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'find'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'find'),
                     mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
             find.return_value = QuerySet
 
-            TestModel1._find(query.Q(a=1))
+            DefaultModel._find(query.Q(a=1))
 
             find.assert_called_with({'a': 1})
 
     def test__find_sorted(self):
         """Test the `_find()` method with a sort."""
 
+        SortedAscModel = ModelFactory('SortedAscModel', sort=('a',))
+        SortedDescModel = ModelFactory('SortedDescModel', sort=('-a',))
+
         with mock.patch('simon.base.QuerySet') as QuerySet:
-            with mock.patch.object(TestModel1._meta.db, 'find') as find:
+            with mock.patch.object(DefaultModel._meta.db, 'find') as find:
                 find.return_value = [{'_id': AN_OBJECT_ID}]
 
-                TestModel1._find()
+                DefaultModel._find()
                 QuerySet.sort.assert_not_called()
 
-            with mock.patch.object(TestModel3._meta.db, 'find') as find:
+            with mock.patch.object(SortedAscModel._meta.db, 'find') as find:
                 find.return_value = [{'_id': AN_OBJECT_ID}]
 
-                TestModel3._find()
+                SortedAscModel._find()
                 QuerySet().sort.assert_called_with('a')
 
-            with mock.patch.object(TestModel4._meta.db, 'find') as find:
+            with mock.patch.object(SortedDescModel._meta.db, 'find') as find:
                 find.return_value = [{'_id': AN_OBJECT_ID}]
 
-                TestModel4._find()
+                SortedDescModel._find()
                 QuerySet().sort.assert_called_with('-a')
 
     def test__update(self):
         """Test the `_update()` method."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1._meta.db, 'update') as update:
+        with mock.patch.object(DefaultModel._meta.db, 'update') as update:
             m._update({'a': 1})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
@@ -295,10 +266,10 @@ class TestDatabase(unittest.TestCase):
     def test__update_atomic(self):
         """Test the `_update()` method with an atomic update."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'update'),
-                    mock.patch.object(TestModel1._meta.db, 'find_one'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
                     ) as (update, find_one):
             find_one.return_value = {'_id': AN_OBJECT_ID, 'a': 1}
 
@@ -315,10 +286,10 @@ class TestDatabase(unittest.TestCase):
         ("Test the `_update()` method with an atomic update of an "
          "embedded document.")
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'update'),
-                    mock.patch.object(TestModel1._meta.db, 'find_one'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
                     ) as (update, find_one):
             find_one.return_value = {'_id': AN_OBJECT_ID, 'a': {'b': 1}}
 
@@ -334,9 +305,9 @@ class TestDatabase(unittest.TestCase):
     def test__update_atomic_unset(self):
         """Test the `_update()` method with `$unset`."""
 
-        m = TestModel1(_id=AN_OBJECT_ID, a=2)
+        m = DefaultModel(_id=AN_OBJECT_ID, a=2)
 
-        with mock.patch.object(TestModel1._meta.db, 'update') as update:
+        with mock.patch.object(DefaultModel._meta.db, 'update') as update:
             m._update({'$unset': {'a': 1}})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
@@ -348,9 +319,9 @@ class TestDatabase(unittest.TestCase):
         ("Test the `_update()` method with `$unset` with an embedded "
          "document.")
 
-        m = TestModel1(_id=AN_OBJECT_ID, a__b=2)
+        m = DefaultModel(_id=AN_OBJECT_ID, a__b=2)
 
-        with mock.patch.object(TestModel1._meta.db, 'update') as update:
+        with mock.patch.object(DefaultModel._meta.db, 'update') as update:
             m._update({'$unset': {'a.b': 1}})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
@@ -361,9 +332,9 @@ class TestDatabase(unittest.TestCase):
     def test__update_field_map(self):
         """Test the `_update()` method with a name in `field_map`."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = MappedModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1._meta.db, 'update') as update:
+        with mock.patch.object(MappedModel._meta.db, 'update') as update:
             m._update({'fake': 1})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
@@ -373,10 +344,10 @@ class TestDatabase(unittest.TestCase):
         ("Test the `_update()` method with a name in `field_map` with "
          "an atomic update.")
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = MappedModel(_id=AN_OBJECT_ID)
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'update'),
-                    mock.patch.object(TestModel1._meta.db, 'find_one'),
+        with nested(mock.patch.object(MappedModel._meta.db, 'update'),
+                    mock.patch.object(MappedModel._meta.db, 'find_one'),
                     ) as (update, find_one):
             find_one.return_value = {'real': 1}
 
@@ -392,23 +363,23 @@ class TestDatabase(unittest.TestCase):
     def test__update_insert(self):
         """Test the `_update()` method for an insert."""
 
-        m = TestModel1()
+        m = DefaultModel()
 
-        with mock.patch.object(TestModel1._meta.db, 'insert') as insert:
+        with mock.patch.object(DefaultModel._meta.db, 'insert') as insert:
             insert.return_value = AN_OBJECT_ID
 
             m._update({'a': 1}, upsert=True)
 
-            insert.assert_called_with(document={'a': 1}, **wc_on)
+            insert.assert_called_with(doc_or_docs={'a': 1}, **wc_on)
 
             self.assertEqual(m._document['_id'], AN_OBJECT_ID)
 
     def test__update_nested_field(self):
         """Test the `_update()` method with an embedded document."""
 
-        m = TestModel1(_id=AN_OBJECT_ID)
+        m = DefaultModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel1._meta.db, 'update') as update:
+        with mock.patch.object(DefaultModel._meta.db, 'update') as update:
             m._update({'a__b': 1})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
@@ -417,9 +388,9 @@ class TestDatabase(unittest.TestCase):
     def test__update_required_field(self):
         """Test the `_update()` method with a required field."""
 
-        m = TestModel5(_id=AN_OBJECT_ID)
+        m = RequiredModel(_id=AN_OBJECT_ID)
 
-        with mock.patch.object(TestModel5._meta.db, 'update') as update:
+        with mock.patch.object(RequiredModel._meta.db, 'update') as update:
             m._update({'a': 1, 'b': 2})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
@@ -429,10 +400,10 @@ class TestDatabase(unittest.TestCase):
         ("Test the `_update()` method with a required field with an "
          "atomic update.")
 
-        m = TestModel5(_id=AN_OBJECT_ID)
+        m = RequiredModel(_id=AN_OBJECT_ID)
 
-        with nested(mock.patch.object(TestModel5._meta.db, 'update'),
-                    mock.patch.object(TestModel5._meta.db, 'find_one'),
+        with nested(mock.patch.object(RequiredModel._meta.db, 'update'),
+                    mock.patch.object(RequiredModel._meta.db, 'find_one'),
                     ) as (update, find_one):
             find_one.return_value = {'a': 1, 'b': 2}
 
@@ -446,10 +417,15 @@ class TestDatabase(unittest.TestCase):
         ("Test the `_update()` method with a required field with an "
          "embedded document.")
 
-        m = TestModel6(_id=AN_OBJECT_ID)
+        RequiredEmbeddedModel = ModelFactory('RequiredModel',
+                                             required_fields=('a.b',))
 
-        with nested(mock.patch.object(TestModel6._meta.db, 'update'),
-                    mock.patch.object(TestModel6._meta.db, 'find_one'),
+        m = RequiredEmbeddedModel(_id=AN_OBJECT_ID)
+
+        with nested(mock.patch.object(RequiredEmbeddedModel._meta.db,
+                                      'update'),
+                    mock.patch.object(RequiredEmbeddedModel._meta.db,
+                                      'find_one'),
                     ) as (update, find_one):
             find_one.return_value = {'a.b': 1}
 
@@ -462,9 +438,9 @@ class TestDatabase(unittest.TestCase):
         ("Test the `_update()` method with a required field with "
          "`$unset`.")
 
-        m = TestModel5(_id=AN_OBJECT_ID, a=1, b=2, c=3)
+        m = RequiredModel(_id=AN_OBJECT_ID, a=1, b=2, c=3)
 
-        with mock.patch.object(TestModel5._meta.db, 'update') as update:
+        with mock.patch.object(RequiredModel._meta.db, 'update') as update:
             m._update({'$unset': {'c': 1}})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
@@ -473,56 +449,60 @@ class TestDatabase(unittest.TestCase):
     def test__update_typeerror(self):
         """Test that `_update()` raises `TypeError`."""
 
-        m1 = TestModel1()
+        m1 = DefaultModel()
 
         # TypeError should be raised when there's no _id.
         with self.assertRaises(TypeError) as e:
             m1._update({})
 
-        expected = ("The 'TestModel1' object cannot be updated because its "
+        expected = ("The 'DefaultModel' object cannot be updated because its "
                     "'_id' attribute has not been set.")
-        actual = e.exception.message
+        actual = str(e.exception)
         self.assertEqual(actual, expected)
 
-        m2 = TestModel5(_id=AN_OBJECT_ID)
-        m3 = TestModel5(_id=AN_OBJECT_ID, a=1, b=2)
+        m2 = RequiredModel(_id=AN_OBJECT_ID)
+        m3 = RequiredModel(_id=AN_OBJECT_ID, a=1, b=2)
 
         # TypeError should be raised when required fields are missing.
         with self.assertRaises(TypeError) as e:
             m2._update({})
 
-        expected = ("The 'TestModel5' object cannot be updated because it must"
-                    " contain all of the required fields: a, b.")
-        actual = e.exception.message
+        expected = ("The 'RequiredModel' object cannot be updated because it "
+                    "must contain all of the required fields: a, b.")
+        actual = str(e.exception)
         self.assertEqual(actual, expected)
 
         with self.assertRaises(TypeError) as e:
             m3._update({'$unset': {'a': 1}})
 
-        expected = ("The 'TestModel5' object cannot be updated because it must"
-                    " contain all of the required fields: a, b.")
-        actual = e.exception.message
+        expected = ("The 'RequiredModel' object cannot be updated because it "
+                    "must contain all of the required fields: a, b.")
+        actual = str(e.exception)
         self.assertEqual(actual, expected)
 
     def test__update_use_internal(self):
         """Test the `_update()` method with `use_internal`."""
 
-        m = TestModel1(_id=AN_OBJECT_ID, a=5)
+        m = DefaultModel(_id=AN_OBJECT_ID, a=5)
 
-        with mock.patch.object(TestModel1._meta.db, 'update') as update:
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
             m._update({'a': 1}, use_internal=True)
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
                                       document={'a': 5}, **wc_on)
 
+            find_one.assert_not_called()
+
     def test__update_use_internal_atomic(self):
         ("Test the `_update()` method with `use_internal` with an "
          "atomic update.")
 
-        m = TestModel1(_id=AN_OBJECT_ID, a__b=5)
+        m = DefaultModel(_id=AN_OBJECT_ID, a__b=5)
 
-        with nested(mock.patch.object(TestModel1._meta.db, 'update'),
-                    mock.patch.object(TestModel1._meta.db, 'find_one'),
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
                     ) as (update, find_one):
             find_one.return_value = {'_id': AN_OBJECT_ID, 'a': {'b': 5}}
 
